@@ -101,6 +101,9 @@ land_geometry = function(dem,inland_proximity,returnpoints=FALSE){
 #' @param betaModel empirical model for TC shape parameter beta (dimensionless Beta)
 #' @param eP background environmental pressure (hPa)
 #' @param rho air density
+#' @param RMAX If params rMaxModel value is NA, use input TC$RMAX
+#' @param VMAX If params rMaxModel value is NA, use input TC$VMAX
+#' @param B If params rMaxModel value is NA, use input TC$B
 #'
 #' @return list of track data inclining the rMax vMax and Beta.
 #' @export
@@ -113,10 +116,13 @@ land_geometry = function(dem,inland_proximity,returnpoints=FALSE){
 #' require(terra)
 #' TCi <- vect(system.file("extdata/YASI/YASI.shp", package="TCHazaRds"))
 #' TCi$PRES <- TCi$BOM_PRES
+#' TCi$RMAX <- TCi$BOM_RMW/1.852 #convert from nautical miles to km
+#' TCi$VMAX <- TCi$BOM_WIND*1.94 #convert from knots to m/s
+#' TCi$B <- 1.4
 #' t1 <- strptime("2011-02-01 09:00:00","%Y-%m-%d %H:%M:%S", tz = "UTC") #first date in POSIX format
 #' t2 <- strptime(rev(TCi$ISO_TIME)[1],"%Y-%m-%d %H:%M:%S", tz = "UTC") #last date in POSIX format
 #' outdate <- seq(t1,t2,"hour") #array sequence from t1 to t2 stepping by “hour”
-
+#' # defult along track parameters are calculated
 #' TCil = update_Track(outdate = outdate,
 #'                    indate = strptime(TCi$ISO_TIME,"%Y-%m-%d %H:%M:%S", tz = "UTC"),
 #'                    TClons = TCi$LON,
@@ -128,18 +134,42 @@ land_geometry = function(dem,inland_proximity,returnpoints=FALSE){
 #'                    vMaxModel=params$vMaxModel,
 #'                    betaModel=params$betaModel,
 #'                    eP = params$eP,
-#'                    rho = params$rhoa)
-#'
+#'                    rho = params$rhoa,
+#'                    RMAX = TCi$RMAX,
+#'                    VMAX = TCi$VMAX,
+#'                    B = TCi$B
+#'                    )
+#' # 'observed' along tack parameters are calculated (#Model = NA)                   
+#' TCil = update_Track(outdate = outdate,
+#'                    indate = strptime(TCi$ISO_TIME,"%Y-%m-%d %H:%M:%S", tz = "UTC"),
+#'                    TClons = TCi$LON,
+#'                    TClats = TCi$LAT,
+#'                    vFms=TCi$STORM_SPD,
+#'                   thetaFms=TCi$thetaFm,
+#'                    cPs=TCi$PRES,
+#'                   rMaxModel=NA,
+#'                    vMaxModel=NA,
+#'                    betaModel=NA,
+#'                    eP = params$eP,
+#'                    rho = params$rhoa,
+#'                    RMAX = TCi$RMAX,
+#'                    VMAX = TCi$VMAX,
+#'                    B = TCi$B
+#'                    )
 update_Track <- function(outdate = NULL, indate, TClons, TClats, vFms, thetaFms, cPs,
-                         rMaxModel, vMaxModel, betaModel, eP, rho = NULL) {
+                         rMaxModel, vMaxModel, betaModel, eP, rho = NULL,RMAX,VMAX,B) {
   TRACK <- list()
   TRACK$eP <- eP
   TRACK$rho <- rho
+  if(is.na(rMaxModel) & is.null(RMAX)) stop("If params rMaxModel value is NA, please provide TC$RMAX")
+  if(is.na(vMaxModel) & is.null(VMAX)) stop("If params vMaxModel value is NA, please provide TC$VMAX")
+  if(is.na(betaModel) & is.null(B)) stop("If params betaModel value is NA, please provide TC$B")
+  
 
   odatei <- as.numeric(indate)
-
+  indatei <- as.numeric(indate)
   if (!is.null(outdate[1])) { # interpolate track to out time steps with approx fun
-    indatei <- as.numeric(indate)
+    
     outdatei <- as.numeric(outdate)
     odatei <- stats::approx(indatei, indatei, outdatei)$y
     TClons <- stats::approx(indatei, TClons, outdatei)$y
@@ -176,8 +206,12 @@ update_Track <- function(outdate = NULL, indate, TClons, TClats, vFms, thetaFms,
     TRACK$rMax <- rMaxModel # use the input values
   }
   if (length(rMaxModel) == 1) {
-    TRACK$rMax <- rMax_modelsR(rMaxModel = rMaxModel, TClats = TRACK$TClats, cPs = TRACK$cPs, eP = eP,
+    if(!is.na(rMaxModel)) if(rMaxModel <= 4) TRACK$rMax <- rMax_modelsR(rMaxModel = rMaxModel, TClats = TRACK$TClats, cPs = TRACK$cPs, eP = eP,
                                dPdt = TRACK$dPdt, vFms = TRACK$vFms, rho = rho)
+    if(is.na(rMaxModel)) {
+      if (is.null(outdate[1])) TRACK$rMax <- RMAX
+      if (!is.null(outdate[1])) TRACK$rMax <- stats::approx(indatei, RMAX, outdatei)$y[s]
+    }
   }
 
   # Compute the Coriolis parameter
@@ -190,17 +224,26 @@ update_Track <- function(outdate = NULL, indate, TClons, TClats, vFms, thetaFms,
     TRACK$vMax <- vMaxModel
   }
   if (length(vMaxModel) == 1) {
-    TRACK$vMax <- vMax_modelsR(vMaxModel = vMaxModel, cPs = TRACK$cPs, eP = eP, vFms = TRACK$vFms,
+    if(!is.na(vMaxModel)) if(vMaxModel <= 4) TRACK$vMax <- vMax_modelsR(vMaxModel = vMaxModel, cPs = TRACK$cPs, eP = eP, vFms = TRACK$vFms,
                                TClats = TRACK$TClats, dPdt = TRACK$dPdt, beta = 1.3, rho = rho) # Holland 80 beta = 1.3
+    if(is.na(vMaxModel)) {
+      if(is.null(outdate[1])) TRACK$vMax <- VMAX
+      if(!is.null(outdate[1])) TRACK$vMax <- stats::approx(indatei, VMAX, outdatei)$y[s]
+    }
   }
 
   # Beta parameter model selection
   if (length(betaModel) > 1) {
     beta <- betaModel
   }
-  TRACK$beta <- beta_modelsR(betaModel = betaModel, vMax = TRACK$vMax, rMax = TRACK$rMax, cPs = TRACK$cPs,
+  if (length(betaModel) == 1) {
+    if(!is.na(betaModel)) if(betaModel <= 4) TRACK$beta <- beta_modelsR(betaModel = betaModel, vMax = TRACK$vMax, rMax = TRACK$rMax, cPs = TRACK$cPs,
                              eP = TRACK$eP, vFms = TRACK$vFms, TClats = TRACK$TClats, dPdt = TRACK$dPdt)
-
+    if(is.na(betaModel)) {
+      if(is.null(outdate[1]))TRACK$beta <- B
+      if(!is.null(outdate[1])) TRACK$beta <- stats::approx(indatei, B, outdatei)$y[s]
+    }
+  }
   return(TRACK)
 }
 
@@ -238,7 +281,9 @@ TCvectInterp = function(outdate = NULL, TC, paramsTable) {
   TRACK <- update_Track(outdate = outdate, indate = indate, TClons = TC$LON, TClats = TC$LAT,
                         vFms = TC$STORM_SPD, thetaFms = TC$thetaFm, cPs = TC$PRES,
                         rMaxModel = params$rMaxModel, vMaxModel = params$vMaxModel,
-                        betaModel = params$betaModel, eP = params$eP, rho = params$rhoa)
+                        betaModel = params$betaModel, eP = params$eP, rho = params$rhoa,
+                        RMAX = TC$RMAX,VMAX = TC$VMAX,B = TC$B
+                        )
 
   # Create a spatial vector from the interpolated track data
   v <- terra::vect(cbind(TRACK$TClons, TRACK$TClats))
@@ -263,6 +308,7 @@ TCvectInterp = function(outdate = NULL, TC, paramsTable) {
 #' @param GEO_land dataframe hazard geometry generated with land_geometry
 #' @param TC SpatVector of Tropical cyclone track parameters
 #' @param paramsTable Global parameters to compute TC Hazards.
+#' @param returnWaves Return ocean wave parameters (default = FALSE)
 #'
 #' @return list() containing a timeseries
 #'
@@ -281,6 +327,9 @@ TCvectInterp = function(outdate = NULL, TC, paramsTable) {
 #' | CP      | TC central Pressure  |  hPa  |
 #' | dPdt      | change in TC CP per hour  |  hPa/hr  |
 #' | vFm      | velocity of TC forward motion  |  m/s  |
+#' | Hs0    | Deep water significant wave height | m |
+#' | Tp0    | Deep water Peak wave period | s |
+#' | Dp0    | The peak direction in which wave are heading | deg clockwise from true north. |
 #'
 #' @details The function calculates wind speed and direction time series from a tropical cyclone track using various wind profile models.
 #' @md
@@ -298,7 +347,7 @@ TCvectInterp = function(outdate = NULL, TC, paramsTable) {
 #' HAZts = TCHazaRdsWindTimeSereies(GEO_land=GEO_land,TC=TCi,paramsTable = paramsTable)
 #' main =  paste(TCi$NAME[1],TCi$SEASON[1],"at",GEO_land$lons,GEO_land$lats)
 #' #with(HAZts,plot(date,Sw,format = "%b-%d %H",type="l",main = main,ylab = "Wind speed [m/s]"))
-TCHazaRdsWindTimeSereies <- function(outdate = NULL, GEO_land = NULL, TC, paramsTable) {
+TCHazaRdsWindTimeSereies <- function(outdate = NULL, GEO_land = NULL, TC, paramsTable,returnWaves = FALSE) {
   # Extract parameters from the paramsTable and convert them into a data frame.
   params <- array(paramsTable$value, dim = c(1, length(paramsTable$value)))
   colnames(params) <- paramsTable$param
@@ -311,7 +360,8 @@ TCHazaRdsWindTimeSereies <- function(outdate = NULL, GEO_land = NULL, TC, params
   TRACK <- update_Track(outdate = outdate, indate = indate, TClons = TC$LON, TClats = TC$LAT,
                         vFms = TC$STORM_SPD, thetaFms = TC$thetaFm, cPs = TC$PRES,
                         rMaxModel = params$rMaxModel, vMaxModel = params$vMaxModel,
-                        betaModel = params$betaModel, eP = params$eP, rho = params$rhoa)
+                        betaModel = params$betaModel, eP = params$eP, rho = params$rhoa,
+                        RMAX = TC$RMAX,VMAX = TC$VMAX,B = TC$B)
 
   # Extract geographical land information
   lon <- GEO_land$lons
@@ -320,7 +370,7 @@ TCHazaRdsWindTimeSereies <- function(outdate = NULL, GEO_land = NULL, TC, params
   # Calculate distance from the center to each grid point
   Rlam <- with(TRACK, RdistPi(Gridlon = lon, Gridlat = lat, TClon = TClons, TClat = TClats))
   R <- Rlam[, 1]
-
+  lam <- Rlam[,2]
   # Calculate pressure based on pressure profile model
   if (params$pressureProfileModel == 0)
     P <- with(TRACK, HollandPressureProfilePi(rMax = rMax, dP = dPs, cP = cPs, beta = beta, R = R))
@@ -342,11 +392,11 @@ TCHazaRdsWindTimeSereies <- function(outdate = NULL, GEO_land = NULL, TC, params
 
   # Calculate wind vortex model
   if (params$windVortexModel == 0)
-    UV <- with(TRACK, KepertWindFieldPi(rMax = rMax, vMax = vMax, vFm = vFms, thetaFm = thetaFms, f = fs, Rlam = Rlam, VZ = VZ))
+    UV <- with(TRACK, KepertWindFieldPi(rMax = rMax, vMax = vMax, vFm = vFms, thetaFm = thetaFms, f = fs, Rlam = Rlam, VZ = VZ, surface = params$surface))
   if (params$windVortexModel == 1)
-    UV <- with(TRACK, HubbertWindFieldPi(rMax = rMax, vFm = vFms, thetaFm = thetaFms, f = fs, Rlam = Rlam, V = V))
+    UV <- with(TRACK, HubbertWindFieldPi(rMax = rMax, vFm = vFms, thetaFm = thetaFms, f = fs, Rlam = Rlam, V = V, surface = params$surface))
   if (params$windVortexModel == 2)
-    UV <- with(TRACK, McConochieWindFieldPi(rMax = rMax, vMax = vMax, vFm = vFms, thetaFm = thetaFms, Rlam = Rlam, V = V, f = fs[1]))
+    UV <- with(TRACK, McConochieWindFieldPi(rMax = rMax, vMax = vMax, vFm = vFms, thetaFm = thetaFms, Rlam = Rlam, V = V, f = fs[1], surface = params$surface))
 
   Uw <- UV[, 1]
   Vw <- UV[, 2]
@@ -370,7 +420,35 @@ TCHazaRdsWindTimeSereies <- function(outdate = NULL, GEO_land = NULL, TC, params
 
   Uw <- cos(Va) * Sw
   Vw <- sin(Va) * Sw
-
+  if(returnWaves){  
+    # significant wave heights OGrady 2024 JCR eq 1
+    Sww = Sw
+    Sww[R < TRACK$rMax & Sww < 10] = 10 #lift winds in the eye so there are always waves
+    dP = params$eP-TRACK$PRES
+    Hs0 = params$Wave_a*Sww^params$Wave_x + params$Wave_b*TRACK$dPs + params$Wave_c*TRACK$vFm
+    Hs0[Hs0 < 0] = 0
+    Hs0[GEO_land$dem > 0] = NA
+    Hs0[R < TRACK$rMax & Hs0 < 2] = 2
+    # wave period Young 2017 eq 4
+    e0 = 96.04*(Hs0^2/16)/(Sww^4)
+    nu0 = (e0/6.365e-6)^-0.303
+    Tp0 = Sww/(nu0*9.8)
+    
+    #Wave directions
+    fs = sign(TRACK$f[1])
+    d = fs*c(0.0,   22.5,   67.5,  112.5,  157.5,180,-180, -157.5, -112.5,  -67.5,  -22.5,  -0.01)
+    #Wave ang outward from the wind Tamizi & Young 2020 (Pers comms)
+    TRACK$thetaFm[TRACK$thetaFm < -180] =  360+TRACK$thetaFm[TRACK$thetaFm < -180]
+    TRACK$thetaFm[TRACK$thetaFm > 180] = -360+TRACK$thetaFm[TRACK$thetaFm >= 180]
+    wwang = fs*c(54,52,36,27,16,56,56,92,104,79,56,54)
+    ang = (90.0-lam) - (90-TRACK$thetaFm) #orientate clockwise to the forward motion thetaFm in degrees
+    ang[ang < -180] =  360+ang[ang < -180]
+    ang[ang >= 180] = -360+ang[ang >= 180]
+    p2a = approx(d,wwang,ang)$y
+    Dp0 = Dw-180+p2a
+    Dp0[Dp0 < 0] = 360 + Dp0[Dp0 < 0]
+    Dp0[GEO_land$dem > 0] = NA
+  }
   # Create and populate the output list
   v <- list() # time series list
   v$date <- TRACK$odatei + strptime("1970-01-01 00:00", "%Y-%m-%d %H:%M", tz = "UTC")
@@ -380,7 +458,7 @@ TCHazaRdsWindTimeSereies <- function(outdate = NULL, GEO_land = NULL, TC, params
   v$Dw <- Dw
   v$P <- P
   v$R <- R
-  v$lam <- Rlam[,2]
+  v$lam <- lam
   v$rMax <- TRACK$rMax
   v$vMax <- TRACK$vMax
   v$beta <- TRACK$beta
@@ -389,7 +467,11 @@ TCHazaRdsWindTimeSereies <- function(outdate = NULL, GEO_land = NULL, TC, params
   v$TClat <- TRACK$TClats
   v$vFm <- TRACK$vFm
   v$thetaFm <- TRACK$thetaFm
-
+  if(returnWaves){
+    v$Hs0 <- Hs0
+    v$Tp0 <- Tp0
+   v$Dp0 <- Dp0
+  }
   return(v)
 }
 
@@ -398,6 +480,7 @@ TCHazaRdsWindTimeSereies <- function(outdate = NULL, GEO_land = NULL, TC, params
 #' @param GEO_land SpatVector or dataframe hazard geometry generated with land_geometry
 #' @param TC SpatVector or data.frame of Tropical cyclone track parameters for a single time step.
 #' @param paramsTable Global parameters to compute TC Hazards.
+#' @param returnWaves Return ocean wave parameters (default = FALSE)
 #'
 #' @return SpatRaster with the following attributes
 #'
@@ -406,10 +489,13 @@ TCHazaRdsWindTimeSereies <- function(outdate = NULL, GEO_land = NULL, TC, params
 #' | abbreviated attribute       | description     | units |
 #' | ------------- | -------------|  -------------|
 #' | P      | Atmospheric pressure | hPa  |
-#' | Uw      | Meridional  wind speed | m/s |
-#' | Vw      | Zonal wind speed | m/s  |
+#' | Uw     | Meridional  wind speed | m/s |
+#' | Vw     | Zonal wind speed | m/s  |
 #' | Sw     | Wind speed | m/s  |
 #' | Dw     | The direction from which wind originates | deg clockwise from true north.   |
+#' | Hs0    | Deep water significant wave height | m |
+#' | Tp0    | Deep water Peak wave period | s |
+#' | Dp0    | The peak direction in which wave are heading | deg clockwise from true north. |
 #'
 #' @md
 #'
@@ -424,7 +510,9 @@ TCHazaRdsWindTimeSereies <- function(outdate = NULL, GEO_land = NULL, TC, params
 #'
 #' TCi = vect(cbind(c(154,154),c(-26.1,-26)),"lines",crs="epsg:4283") #track line segment
 #' TCi$PRES = 950
-#' TCi$RMW = 40
+#' TCi$RMAX = 40
+#' TCi$VMAX = 60
+#' TCi$B = 1.4
 #' TCi$ISO_TIME = "2022-10-04 20:00:00"
 #' TCi$LON = geom(TCi)[1,3]
 #' TCi$LAT = geom(TCi)[1,4]
@@ -447,7 +535,7 @@ TCHazaRdsWindTimeSereies <- function(outdate = NULL, GEO_land = NULL, TC, params
 #' #vectorplot(UV, isField='dXY', col.arrows='white', aspX=0.002,aspY=0.002,at=ats ,
 #' #colorkey=list( at=ats), par.settings=viridisTheme)
 #'
-TCHazaRdsWindField <- function(GEO_land, TC, paramsTable) {
+TCHazaRdsWindField <- function(GEO_land, TC, paramsTable,returnWaves = FALSE) {
   # Extract parameters from paramsTable
   params <- array(paramsTable$value, dim = c(1, length(paramsTable$value)))
   colnames(params) <- paramsTable$param
@@ -459,7 +547,7 @@ TCHazaRdsWindField <- function(GEO_land, TC, paramsTable) {
     # Reformat track data
     TRACK <- update_Track(outdate = NULL, indate = indate, TClons = TC$LON, TClats = TC$LAT, vFms = TC$STORM_SPD, thetaFms = TC$thetaFm,
                           cPs = TC$PRES, rMaxModel = params$rMaxModel, vMaxModel = params$vMaxModel, betaModel = params$betaModel,
-                          eP = params$eP, rho = params$rhoa)
+                          eP = params$eP, rho = params$rhoa,RMAX = TC$RMAX,VMAX = TC$VMAX,B = TC$B)
   } else if (methods::is(TC, "data.frame")) {
     TRACK <- TC
     indate <- strptime("1970-01-01 00:00:00", "%Y-%m-%d %H:%M:%S", tz = "UTC") + TRACK$odatei
@@ -544,6 +632,41 @@ TCHazaRdsWindField <- function(GEO_land, TC, paramsTable) {
   Uw <- cos(Va) * Sw
   Vw <- sin(Va) * Sw
 
+  R <- ept
+  terra::values(R) <- Rlam[,1]
+  lam <- ept
+  terra::values(lam) <- Rlam[,2]
+  if(returnWaves){  
+    # significant wave heights OGrady 2024 JCR eq 1
+    Sww = Sw
+    Sww[R < TRACK$rMax & Sww < 10] = 10 #lift winds in the eye so there are always waves
+    dP = params$eP-TRACK$PRES
+    Hs0 = params$Wave_a*Sww^params$Wave_x + params$Wave_b*TRACK$dPs + params$Wave_c*TRACK$vFm
+    Hs0[Hs0 <0] = 0
+    Hs0[GEO_land$dem > 0] = NA
+    Hs0[R < TRACK$rMax & Hs0 < 2] = 2
+    # wave period Young 2017 eq 4
+    e0 = 96.04*(Hs0^2/16)/(Sww^4)
+    nu0 = (e0/6.365e-6)^-0.303
+    Tp0 = Sww/(nu0*9.8)
+    
+    #Wave directions
+    fs = sign(TRACK$f)
+    d = fs*c(0.0,   22.5,   67.5,  112.5,  157.5,180,-180, -157.5, -112.5,  -67.5,  -22.5,  -0.01)
+    #Wave ang outward from the wind Tamizi & Young 2020 (Pers comms)
+    TRACK$thetaFm[TRACK$thetaFm < -180] =  360+TRACK$thetaFm[TRACK$thetaFm < -180]
+    TRACK$thetaFm[TRACK$thetaFm > 180] = -360+TRACK$thetaFm[TRACK$thetaFm >= 180]
+    wwang = fs*c(54,52,36,27,16,56,56,92,104,79,56,54)
+    ang = (90.0-lam) - (90-TRACK$thetaFm) #orientate clockwise to the forward motion thetaFm in degrees
+    ang[ang < -180] =  360+ang[ang < -180]
+    ang[ang >= 180] = -360+ang[ang >= 180]
+    p2av = approx(d,wwang,values(ang))$y
+    p2a <- ept
+    terra::values(p2a) <- p2av
+    Dp0 = Dw-180+p2a
+    Dp0[Dp0 < 0] = 360 + Dp0[Dp0 < 0]
+    Dp0[GEO_land$dem > 0] = NA
+  }
   # Create a raster stack with wind field components
   terra::time(Pr) <- indate
   terra::units(Pr) <- "hPa"
@@ -564,11 +687,31 @@ TCHazaRdsWindField <- function(GEO_land, TC, paramsTable) {
   terra::time(Dw) <- indate
   terra::units(Dw) <- "Deg"
   terra::longnames(Dw) <- "wind_direction"
-
-  rl <- list(Pr, Uw, Vw, Sw, Dw)
-  rout <- terra::rast(rl)
-  names(rout) <- c("Pr", "Uw", "Vw", "Sw", "Dw")
-
+  
+  if(!returnWaves){
+    rl <- list(Pr, Uw, Vw, Sw, Dw)
+    rout <- terra::rast(rl)
+    names(rout) <- c("Pr", "Uw", "Vw", "Sw", "Dw")
+    
+  }
+  
+  if(returnWaves){  
+    terra::time(Hs0) <- indate
+    terra::units(Hs0) <- "m"
+    terra::longnames(Hs0) <- "Deep_water_significant_wave_height"
+    
+    terra::time(Tp0) <- indate
+    terra::units(Tp0) <- "s"
+    terra::longnames(Tp0) <- "peak_wave_period"
+  
+    terra::time(Dp0) <- indate
+    terra::units(Dp0) <- "Deg"
+    terra::longnames(Dp0) <- "peak_wave_direction"
+  
+    rl <- list(Pr, Uw, Vw, Sw, Dw, Hs0, Tp0,Dp0)
+    rout <- terra::rast(rl)
+    names(rout) <- c("Pr", "Uw", "Vw", "Sw", "Dw","Hs0", "Tp0","Dp0")
+  }
   return(rout)
 }
 
@@ -580,6 +723,8 @@ TCHazaRdsWindField <- function(GEO_land, TC, paramsTable) {
 #' @param paramsTable Global parameters to compute TC Hazards
 #' @param outfile character. Output netcdf filename
 #' @param overwrite TRUE/FALSE, option to overwrite outfile
+#' @param returnWaves Return ocean wave parameters (default = FALSE)
+#' 
 #' @return SpatRasterDataset with the following attributes.
 #'
 #'
@@ -591,6 +736,9 @@ TCHazaRdsWindField <- function(GEO_land, TC, paramsTable) {
 #' | Vw      | Zonal wind speed | m/s  |
 #' | Sw     | Wind speed | m/s  |
 #' | Dw     | The direction from which wind originates | deg clockwise from true north  |
+#' | Hs0    | Deep water significant wave height | m |
+#' | Tp0    | Deep water Peak wave period | s |
+#' | Dp0    | The peak direction in which wave are heading | deg clockwise from true north. |
 #'
 #' @md
 #'
@@ -605,7 +753,9 @@ TCHazaRdsWindField <- function(GEO_land, TC, paramsTable) {
 #'
 #' TCi = vect(cbind(c(154,154),c(-26.1,-26)),"lines",crs="epsg:4283") #track line segment
 #' TCi$PRES = 950
-#' TCi$RMW = 40
+#' TCi$RMAX = 40
+#' TCi$VMAX = 60
+#' TCi$B = 1.4
 #' TCi$ISO_TIME = "2022-10-04 20:00:00"
 #' TCi$LON = geom(TCi)[1,3]
 #' TCi$LAT = geom(TCi)[1,4]
@@ -625,7 +775,7 @@ TCHazaRdsWindField <- function(GEO_land, TC, paramsTable) {
 #' HAZi = TCHazaRdsWindFields(outdate=outdate,GEO_land=GEO_land,TC=TC,paramsTable=paramsTable)
 #' plot(min(HAZi$Pr))
 #'
-TCHazaRdsWindFields <- function(outdate = NULL, GEO_land, TC, paramsTable, outfile = NULL, overwrite = FALSE) {
+TCHazaRdsWindFields <- function(outdate = NULL, GEO_land, TC, paramsTable, outfile = NULL, overwrite = FALSE,returnWaves = FALSE) {
   # Extract and format the parameters
   params <- data.frame(t(paramsTable$value))
   colnames(params) <- paramsTable$param
@@ -637,7 +787,8 @@ TCHazaRdsWindFields <- function(outdate = NULL, GEO_land, TC, paramsTable, outfi
   TRACK <- as.data.frame(update_Track(outdate = outdate, indate = indate, TClons = TC$LON, TClats = TC$LAT,
                                       vFms = TC$STORM_SPD, thetaFms = TC$thetaFm, cPs = TC$PRES,
                                       rMaxModel = params$rMaxModel, vMaxModel = params$vMaxModel,
-                                      betaModel = params$betaModel, eP = params$eP, rho = params$rhoa))
+                                      betaModel = params$betaModel, eP = params$eP, rho = params$rhoa,
+                                      RMAX = TC$RMAX,VMAX = TC$VMAX,B = TC$B))
 
   nt <- nrow(TRACK)
   TRACK$PRES <- TRACK$cPs
@@ -648,7 +799,7 @@ TCHazaRdsWindFields <- function(outdate = NULL, GEO_land, TC, paramsTable, outfi
 
   # Compute wind fields for each time step
   s <- which(!is.na(TRACK$cPs))
-  HAZ_l <- lapply(s, function(x) TCHazaRdsWindField(GEO_land = GEO_land, TC = TRACK[x,], paramsTable = paramsTable))
+  HAZ_l <- lapply(s, function(x) TCHazaRdsWindField(GEO_land = GEO_land, TC = TRACK[x,], paramsTable = paramsTable,returnWaves = returnWaves))
 
   # Extract wind field components and create spatial data sets
   Pr <- terra::rast(lapply(HAZ_l, function(x) x$Pr))
@@ -656,18 +807,31 @@ TCHazaRdsWindFields <- function(outdate = NULL, GEO_land, TC, paramsTable, outfi
   Vw <- terra::rast(lapply(HAZ_l, function(x) x$Vw))
   Sw <- terra::rast(lapply(HAZ_l, function(x) x$Sw))
   Dw <- terra::rast(lapply(HAZ_l, function(x) x$Dw))
+  if(returnWaves){
+    Hs0 <- terra::rast(lapply(HAZ_l, function(x) x$Hs0))
+    Tp0 <- terra::rast(lapply(HAZ_l, function(x) x$Tp0))
+    Dp0 <- terra::rast(lapply(HAZ_l, function(x) x$Dp0))
+  }
 
   # Combine spatial data sets into a single sds object
-  HAZs <- terra::sds(list(Pr = Pr, Uw = Uw, Vw = Vw, Sw = Sw, Dw = Dw))
-  terra::varnames(HAZs) <- c("Pr", "Uw", "Vw", "Sw", "Dw")
-  terra::longnames(HAZs) <- c("air_pressure_at_sea_level", "eastward_wind", "northward_wind", "wind_speed", "wind_direction")
-  terra::units(HAZs) <- c("hPa", "m/s", "m/s", "m/s", "m/s")
-
+  if(!returnWaves){
+    HAZs <- terra::sds(list(Pr = Pr, Uw = Uw, Vw = Vw, Sw = Sw, Dw = Dw))
+    terra::varnames(HAZs) <- c("Pr", "Uw", "Vw", "Sw", "Dw")
+    terra::longnames(HAZs) <- c("air_pressure_at_sea_level", "eastward_wind", "northward_wind", "wind_speed", "wind_direction")
+    terra::units(HAZs) <- c("hPa", "m/s", "m/s", "m/s", "deg")
+  }
+  if(returnWaves){
+    HAZs <- terra::sds(list(Pr = Pr, Uw = Uw, Vw = Vw, Sw = Sw, Dw = Dw, Hs0 = Hs0, Tp0 = Tp0, Dp0 = Dp0))
+    terra::varnames(HAZs) <- c("Pr", "Uw", "Vw", "Sw", "Dw","Hs0","Tp0","Dp0")
+    terra::longnames(HAZs) <- c("air_pressure_at_sea_level", "eastward_wind", "northward_wind", "wind_speed", "wind_direction","Deep_water_significant_wave_height","peak_period","peak_wave_direction")
+    terra::units(HAZs) <- c("hPa", "m/s", "m/s", "m/s", "deg","m","s","deg")
+  }
+  
+  
   # Write output to a file if provided
   if (!is.null(outfile)) {
     terra::writeCDF(HAZs, filename = outfile, overwrite = overwrite)
   }
-
   return(HAZs)
 }
 
@@ -723,9 +887,9 @@ rMax_modelsR <- function(rMaxModel, TClats, cPs, eP, R175ms = 150, dPdt = NULL,
   #3: Vickery & Wadhera (2008) eq 11
   #4: Takagi & Wu (2016)
   #5: Chavas & Knaff (2022)
+  if(is.na(rMaxModel)) stop("rMaxModel must be a value from 0 to 5, see params file") 
   dP <- (eP - cPs)
   dP[dP < 1] <- 1
-
   if (rMaxModel == 0) {
     # Powell Soukup et al (2005) updated to Arthur 2021
     rMaxs <- exp(3.543 - 0.00378 * dP + 0.813 * exp(-0.0022 * (dP)^2) + 0.00157 * (TClats^2))
@@ -794,12 +958,13 @@ vMax_modelsR <- function(vMaxModel, cPs, eP, vFms = NULL, TClats = NULL, dPdt =
 
   dP = eP - cPs
   dP[dP < 1] = 1 # Needs to be at least lower than the background
+  if(is.na(vMaxModel)) stop("vMaxModel must be a value from 0 to 4, see params file") 
 
   if (vMaxModel == 0) {
     # see https://github.com/GeoscienceAustralia/tcha/blob/6e6de8df2b3fd5c9287d060f1f7f1d4ff63e87a7/wind-radii/rmax_fit.py#L205
     vMax = 0.6252 * sqrt(dP * 100)
   }
-
+  
   if (vMaxModel == 1) {
     x = 0.6 * (1 - dP / 215)
     bs = -4.4e-5 * dP^2 + 0.01 * dP + 0.03 * dPdt - 0.014 * abs(TClats) + 0.15 * vFms^x + 1
@@ -859,7 +1024,8 @@ beta_modelsR <- function(betaModel, vMax, rMax, cPs, eP, vFms, TClats, dPdt, rho
 
   dP <- (eP - cPs)
   dP[dP < 1] <- 1
-
+  if(is.na(betaModel)) stop("betaModel must be a value from 0 to 4, see params file") 
+  
   if (betaModel == 0) {
     beta <- 1.881093 - 0.010917 * abs(TClats) - 0.005567 * rMax # Powell (2005)
     beta[beta < 0.8] <- 0.8
@@ -921,7 +1087,8 @@ inlandWindDecay = function(d,a = c(0.66,1,0.4)){
 #' require(terra)
 #' TCi <- vect(cbind(c(154.1,154),c(-26.1,-26)),"lines",crs="epsg:4283") #track line segment
 #' TCi$PRES <- 950
-#' TCi$RMW <- 40
+#' TCi$RMAX <- 40
+#' TCi$B <- 1.4
 #' TCi$ISO_TIME <- "2022-10-04 20:00:00"
 #' TCi$LON <- geom(TCi)[1,3]
 #' TCi$LAT <- geom(TCi)[1,4]
@@ -984,7 +1151,9 @@ TCProfilePts = function(TC_line,Through_point=NULL,bear=NULL,length =200,step=2)
 #'
 #' TCi = vect(cbind(c(154,154),c(-26.1,-26)),"lines",crs="epsg:4283") #track line segment
 #' TCi$PRES = 950
-#' TCi$RMW = 40
+#' TCi$RMAX = 40
+#' TCi$VMAX = 60
+#' TCi$B = 1.4
 #' TCi$ISO_TIME = "2022-10-04 20:00:00"
 #' TCi$LON = geom(TCi)[1,3]
 #' TCi$LAT = geom(TCi)[1,4]
@@ -1017,7 +1186,8 @@ TCHazaRdsWindProfile = function(GEO_land,TC,paramsTable){
     indate=strptime(TC$ISO_TIME,"%Y-%m-%d %H:%M:%S",tz = "UTC")
     #reformat
     TRACK = update_Track(outdate=NULL,indate=indate,TClons=TC$LON,TClats=TC$LAT,vFms=TC$STORM_SPD,thetaFms=TC$thetaFm,cPs=TC$PRES,
-                         rMaxModel=params$rMaxModel,vMaxModel=params$vMaxModel,betaModel=params$betaModel,eP = params$eP,rho = params$rhoa)
+                         rMaxModel=params$rMaxModel,vMaxModel=params$vMaxModel,betaModel=params$betaModel,eP = params$eP,rho = params$rhoa,
+                         RMAX = TC$RMAX,VMAX = TC$VMAX,B = TC$B)
   }
   if(methods::is(TC,"data.frame")){
     TRACK = TC
